# 输入文本框(带按钮，按enter也可以运行)
"""
创建(就当button控件用)：
a=inpu_box(tk,函数名,**按钮的配置)

打包：
a.pack(**pack的配置)

配置：
a.config_entry(**文本框的配置)
a.config_button(**按钮的配置)
其实你基本可以当做Entry来用，除了初始化语句还有配置以外基本都是一样的
"""
import tkinter as tk

class inpt_box:
	def __init__(self, *w, command, **arg):
		import tkinter as tk
		self.Frame = tk.Frame(*w)
		self.cmd = command
		self.Entry = tk.Entry(self.Frame, bd=3)
		self.Button = tk.Button(self.Frame, command=self.cmd, **arg)

	def config_entry(self, **arg):
		self.Entry.configure(**arg)

	def config_button(self, **arg):
		self.button.configure(**arg)

	def get(self):
		return self.Entry.get()

	def insert(self, *w):
		self.Entry.insert(*w)

	def delete(self, *w):
		self.Entry.delete(*w)

	def _kao(self, *event):
		self.cmd()

	def pack(self, **arg):
		self.Entry.pack(fill="x", side="left", expand=True)
		self.Button.pack(fill="x", side="left")
		self.Entry.bind("<Return>", self._kao)
		self.Frame.pack(**arg)


# --end--

# 覆盖tk的Text控件
from tkinter.scrolledtext import ScrolledText as Text


# --end---
class Imgbutton:
	def __init__(self, root, width=100, img="", text="", command="", **arg):
		import tkinter as tk
		import PIL.Image
		import PIL.ImageTk
		self.img = PIL.ImageTk.PhotoImage(
			PIL.Image.open(img).resize(
				(width, width),
				PIL.Image.ANTIALIAS
			)
		)
		self.command = command
		self.Frame = tk.Frame(root)
		self.Label = tk.Label(self.Frame, text=text, image=self.img, compound="top", **arg)
		self.Label.bind("<Button-1>", self.command)
		self.Label.pack(fill="both", expand=True)

	# def command(self,*event):
	#     self.com()
	def configure(self, **arg):
		self.Label.configure(**arg)

	def pack(self, *K):
		self.Frame.pack(*K)

class ListBX:
	def __init__(self,root,*w,**arg):
		import tkinter as tk
		self.Frame = tk.Frame(root)
		# self.cmd = command
		self.get = tk.Scrollbar(self.Frame)
		self.ListBox = tk.ListBox(self.Frame, *w,yscrollcommand=self.get.set,**arg)
		self.get.config(command=self.ListBox.yview)
		self.ListBox.pack(side=tk.LEFT, fill=tk.BOTH,expand=True)
		self.get.pack(side=tk.RIGHT,fill=tk.X)
	def config(self,**kwargs):
		self.ListBox.config(**kwargs)
	def run(self,text:"str"):
		exec("self.ListBox."+text)
	def pack(self,*w,**kwargs):
		self.Frame.pack(*w,**kwargs)
class Show_text(tk.Text):
	def __init__(self,*w,**kwargs):
		super().__init__(*w,**kwargs)
		self.configure(exportselection=0,state=tk.DISABLED,relief=tk.RAISED)
	def finsert(self,*w,**kwargs):
		self.configure(exportselection=0, state=tk.NORMAL)
		self.insert(*w,**kwargs)
		self.configure(exportselection=0, state=tk.DISABLED)










# _update = [
# 	{
# 		"ver": "1.0.0",
# 		"text": "这是这个库最初的模样，只能检测点赞"
# 	},
# 	{
# 		"ver": "1.0.1",
# 		"text": "优化了点赞功能"
# 	},
# 	{
# 		"ver": "1.3.2",
# 		"text": "添加了运行程序功能"
# 	},
# 	{
# 		"ver": "1.5.0",
# 		"text": "这个库开始整合，项目小组正式成立"
# 	},
# 	{
# 		"ver": "1.8.7",
# 		"text": "开始加入帮助"
# 	},
# 	{
# 		"ver": "2.0.0",
# 		"text": "加入加载封面功能"
# 	},
# 	{
# 		"ver": "2.0.5",
# 		"text": "优化帮助"
# 	},
# 	{
# 		"ver": "2.1.0",
# 		"text": "加入英文翻译"
# 	},
# 	{
# 		"ver": "2.2.0",
# 		"text": "添加很多小功能"
# 	}
# ]
# _ver = "2.2.0"
# _mit = "Beta"
# _logo = """
# \   / ┌──   ╭───
#  \ /　Ｅ──  │
# 　Ｘ   └──  Ｓ──╮
#  / \  Ａ-Ｐ-Ｐ  │
# /   \       ───╯
# """
class xesapp():
	"与学而思作品相关的工具"
	def __init__(self, pid):
		"加载一个学而思作品，参数pid：学而思作品网址"
		self.pid = pid.split("&pid=")[1].split("&")[0]
		self.updata()
		self.like = self.data["likes"]
		self.unlike = self.data["unlikes"]
		self.last_comments=self.data["comments"]
	def updata(self):
		"更新当前数据"
		import requests
		url = "http://code.xueersi.com/api/compilers/" + self.pid + "?id=" + self.pid
		headers = {'Content-Type': 'application/json'}
		a = requests.get(url=url, headers=headers)
		a = self._nice(a.text).replace("false", "False").replace("true", "True").replace("null", "None")
		z = eval(a)
		# self._nice()
		self.data = z["data"]
	def get_modified(self):
		return self.data["modified_at"]
	def get_published(self):
		return self.data["published_at"]
	def get_view(self):
		"获取观看量"
		return self.data["views"]
	def get_cover(self):
		"获取封面url"
		return self.data["thumbnail"]
	def load_cover(self, savaname):
		"下载封面到savaname"

		def load(url, name):
			import requests as req
			response = req.get(url)
			try:
				a = open(name, "wb")
				a.write(response.content)
				a.close()
			except:
				a = open(name, "w")
				a.write(response.text)
				a.close()

		load(self.data["thumbnail"], savaname)
	def get_hot(self):
		"获取作品热度值"
		return self.data["popular_score"]
	def is_like(self):
		"获取当前用户是否点赞，返回：如果点赞，返回True，如果差评，返回False，否则返回None"
		self.updata()
		like = self.data["likes"]
		unlike = self.data["unlikes"]
		if int(self.like) < int(like):
			return True
		elif int(self.unlike) < int(unlike):
			return False
		else:
			return None
	def get_like(self, updata=False):
		"获取当前点赞数，返回：当前作品的点赞数"
		if updata:
			self.updata()
		return self.data["likes"]
	def get_unlike(self, updata=False):
		"获取当前差评数，返回：当前作品的差评数"
		if updata:
			self.updata()
		return self.data["unlikes"]
	def run_app(self):
		"运行学而思程序"
		self.load_file()
		exec(self.get_code())
	def _nice(self, emoji_str):
		import struct
		return ''.join(
			c if c <= '\uffff' else ''.join(chr(x) for x in struct.unpack('>2H', c.encode('utf-16be'))) for c in
			emoji_str)
	def get_code(self):
		"获取作品代码，返回：作品源代码"
		return self._nice(self.data["xml"])
	def load_file(self, cache=""):
		"加载一个作品的文件"

		def load(url, name):
			import requests as req
			response = req.get(url)
			try:
				a = open(name, "wb")
				a.write(response.content)
				a.close()
			except:
				a = open(name, "w")
				a.write(response.text)
				a.close()

		k = self.data["assets"]["assets"]
		for x in k:
			load(("https://livefile.xesimg.com/programme/python_assets/" + x["md5ext"]), cache + x["name"])
	def is_hidden_code(self):
		if self.data["hidden_code"]==2:
			return True
		else:
			return False
	def get_description(self):
		return self.data["description"]
	def get_comments(self):
		return self.data["comments"]
	def is_comments(self):
		if self.data["comments"]>self.last_comments:
			return True
		else:
			return False

#失效
class cmt:
	def __init__(self,pid):
		self.pid = pid.split("&pid=")[1].split("&")[0]
		self.updata()
		self.dta_dic=self._in_dta(self.data)
	def _nice(self, emoji_str):
		import struct
		return ''.join(
			c if c <= '\uffff' else ''.join(chr(x) for x in struct.unpack('>2H', c.encode('utf-16be'))) for c in
			emoji_str)
	def updata(self):
		"更新当前数据"
		import requests
		s = requests.Session()
		url = "http://code.xueersi.com/api/comments?appid=1001108&topic_id=CP_{pid}&parent_id=0&order_type=&page=1&per_page=15".replace("{pid}",self.pid)
		a = s.get(url=url, headers={'Accept': 'application/json, text/plain, */*', 'Accept-Encoding': 'gzip, deflate',
			  'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6',
			  'Cookie': 'xesId=b524835904a4a420cba3dde34890bade; user-select=scratch;  xes_run_token=eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiIuY29kZS54dWVlcnNpLmNvbSIsImF1ZCI6Ii5jb2RlLnh1ZWVyc2kuY29tIiwiaWF0IjoxNjAxODA5NDcxLCJuYmYiOjE2MDE4MDk0NzEsImV4cCI6MTYwMTgyMzg3MSwidXNlcl9pZCI6bnVsbCwidWEiOiJNb3ppbGxhXC81LjAgKFdpbmRvd3MgTlQgMTAuMDsgV2luNjQ7IHg2NCkgQXBwbGVXZWJLaXRcLzUzNy4zNiAoS0hUTUwsIGxpa2UgR2Vja28pIENocm9tZVwvODUuMC40MTgzLjEyMSBTYWZhcmlcLzUzNy4zNiBFZGdcLzg1LjAuNTY0LjY4IiwiaXAiOiIxMTIuNDkuNzIuMTc1In0.9bXcb813GhSPhoUJkezZpV8O50ynm0hhYvszNyczznQ; prelogid=ef8f6d12febabf75bf9599744b73c6f5; xes-code-id=87f66376f1afd34f70339baeca61b7a1.8dbd833da9122d69a17f91054066dbb3; X-Request-Id=82f1c3968c8ff01ee151a0413f56aa84; Hm_lpvt_a8a78faf5b3e92f32fe42a94751a74f1=1601809487',
			  'Host': 'code.xueersi.com', 'Proxy-Connection': 'keep-alive',
			  'Referer': 'http://code.xueersi.com/space/11909587',
			  'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.121 Safari/537.36 Edg/85.0.564.68'})
		a = self._nice(a.text).replace("false", "False").replace("true", "True").replace("null", "None")
		z = eval(a)
		print(z)
		# self._nice()
		self.data = z["data"]["data"]
	def _in_dta(self,data):
		dta_dic = []
		for x in data:
			kk={}
			kk["user_id"] = x["user_id"]
			kk["content"] = x["content"]
			kk["created_at"] = x["created_at"]
			kk["data"]=kk["created_at"]
			kk["username"]=x["username"]
			kk["reply_username"] = x["reply_username"]
			kk["likes"]=x["likes"]
			z=x["reply_list"]["data"]
			z_list=[]
			for n in z:
				k = {}
				k["user_id"] = n["user_id"]
				k["content"] = n["content"]
				k["created_at"] = n["created_at"]
				k["reply_username"] = n["reply_username"]
				k["username"] = n["username"]
				z_list.append(k)

			kk["reply_list"]=z_list

			dta_dic.append(kk)
		return dta_dic
	def sorted(self,sort_by="likes",reverse=True):
		if sort_by=="likes":
			def sort(z):
				return z["likes"]
		elif sort_by=="data":
			def sort(z):
				return z["data"]
		self.dta_dic=sorted(self.dta_dic,key=sort,reverse=reverse)
		return self.dta_dic
	def fmt(self):
		str=""
		for x in self.dta_dic:
			str+="["+x["created_at"]+"]\n\t"+x["username"]+"回复"+x["reply_username"]+":"+x["content"].replace("\n","\n\t")+"\n"

			for y in x["reply_list"]:
				str += "\t[" + y["created_at"] + "]\n\t\t" + y["username"] + "回复" + y["reply_username"] + ":" + y[
					"content"].replace("\n","\n\t\t") + "\n"
			str += "\n"
		return str
#
#






def load_requests(str,between=":",line="\n"):
	rc={}
	for x in str.split(line):
		if x=="":
			continue
		y=x.split(between)
		if len(y)==1:
			continue
		elif len(y)!=2:
			y[1]="".join(y[1:])
		rc[y[0]]=y[1]
	return rc
ua_txt="""IE6.0:Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)
IE7.0:Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)
IE8.0:Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1)
IE9.0:Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1)
Firefox3.5:Mozilla/5.0 (compatible; rv:1.9.1) Gecko/20090702 Firefox/3.5
Firefox3.6:Mozilla/5.0 (compatible; rv:1.9.2) Gecko/20100101 Firefox/3.6
Firefox4.0:Mozilla/5.0 (compatible; rv:2.0) Gecko/20110101 Firefox/4.0
Firefox6.0:Mozilla/5.0 (Windows NT 6.1; WOW64; rv:6.0.2) Gecko/20100101 Firefox/6.0.2
Chrome11.0:Mozilla/5.0 (compatible) AppleWebKit/534.21 (KHTML, like Gecko) Chrome/11.0.682.0 Safari/534.21
Opera11.0:Opera/9.80 (compatible; U) Presto/2.7.39 Version/11.00
Maxthon3.0:Mozilla/5.0 (compatible; U) AppleWebKit/533.1 (KHTML, like Gecko) Maxthon/3.0.8.2 Safari/533.1
Android:Mozilla/5.0 (Linux; U; Android 2.2) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1 
iPhone:Mozilla/5.0 (iPhone; U; CPU OS 4_2_1 like Mac OS X) AppleWebKit/532.9 (KHTML, like Gecko) Version/5.0.3 Mobile/8B5097d Safari/6531.22.7
iPad:Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/4.0.2 Mobile/8C148 Safari/6533.18.5
Safari5.0:Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7) AppleWebKit/534.16+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4
aa:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.87 Safari/537.36
aa:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.87 Safari/537.36
aa:Mozilla/5.0 (Windows NT 6.1; WOW64; rv:46.0) Gecko/20100101 Firefox/46.0
aa:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.87 Safari/537.36 OPR/37.0.2178.32
aa:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2
aa:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/45.0.2454.101 Safari/537.36
aa:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/45.0.2454.101 Safari/537.36
aa:Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/46.0.2486.0 Safari/537.36 Edge/13.10586
aa:Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; rv:11.0) like Gecko
aa:Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)
aa:Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)
aa:Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0)
aa:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 BIDUBrowser/8.3 Safari/537.36
aa:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Maxthon/4.9.2.1000 Chrome/39.0.2146.0 Safari/537.36
aa:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.80 Safari/537.36 Core/1.47.277.400 QQBrowser/9.4.7658.400
aa:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/48.0.2564.116 UBrowser/5.6.12150.8 Safari/537.36
aa:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/38.0.2125.122 Safari/537.36 SE 2.X MetaSr 1.0
aa:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.154 Safari/537.36 LBBROWSER 
aa:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/48.0.2564.116 Safari/537.36 TheWorld 7
User-Agent:Mozilla/4.0(compatible;MSIE7.0;WindowsNT5.1;AvantBrowser)
User-Agent:Mozilla/4.0(compatible;MSIE7.0;WindowsNT5.1)
User-Agent:Mozilla/5.0(Macintosh;U;IntelMacOSX10_6_8;en-us)AppleWebKit/534.50(KHTML,likeGecko)Version/5.1Safari/534.50
User-Agent:Mozilla/5.0(Windows;U;WindowsNT6.1;en-us)AppleWebKit/534.50(KHTML,likeGecko)Version/5.1Safari/534.50
User-Agent:Mozilla/5.0(compatible;MSIE9.0;WindowsNT6.1;Trident/5.0;
User-Agent:Mozilla/4.0(compatible;MSIE8.0;WindowsNT6.0;Trident/4.0)
User-Agent:Mozilla/4.0(compatible;MSIE7.0;WindowsNT6.0)
User-Agent:Mozilla/4.0(compatible;MSIE6.0;WindowsNT5.1)
User-Agent:Mozilla/5.0(Macintosh;IntelMacOSX10.6;rv:2.0.1)Gecko/20100101Firefox/4.0.1
User-Agent:Mozilla/5.0(WindowsNT6.1;rv:2.0.1)Gecko/20100101Firefox/4.0.1
User-Agent:Opera/9.80(Macintosh;IntelMacOSX10.6.8;U;en)Presto/2.8.131Version/11.11
User-Agent:Opera/9.80(WindowsNT6.1;U;en)Presto/2.8.131Version/11.11
User-Agent:Mozilla/5.0(Macintosh;IntelMacOSX10_7_0)AppleWebKit/535.11(KHTML,likeGecko)Chrome/17.0.963.56Safari/535.11
User-Agent:Mozilla/4.0(compatible;MSIE7.0;WindowsNT5.1;Maxthon2.0)
User-Agent:Mozilla/4.0(compatible;MSIE7.0;WindowsNT5.1;Trident/4.0;SE2.XMetaSr1.0;SE2.XMetaSr1.0;.NETCLR2.0.50727;SE2.XMetaSr1.0)
User-Agent:Mozilla/4.0(compatible;MSIE7.0;WindowsNT5.1;360SE)"""

ua_list=[]
for _x in ua_txt.split("\n"):
	ua_list.append(":".join(_x.split(":")[1:]))
ua_list=list(set(ua_list))
def get_xes_url(url):
	import json,requests
	headers={"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.102 Safari/537.36",'Content-Type': 'application/json'}
	c=requests.get(url,headers=headers)
	b = str(c.status_code)
	if b[:2] not in ["20","30"]:
		import xingyunlib.err
		xingyunlib.err.err("爬取失败！")
	else:
		return json.loads(c.text)
def spider(*args,**kwargs):
	import requests
	a=requests.get(*args,**kwargs)
	b=str(a.status_code)
	if b[:2] not in ["20","30"]:
		import xingyunlib.err
		xingyunlib.err.err(f"爬取错误，状态码：{b}")
		return
	return a




import os
def cre():
	a=input("文件夹名称:")
	global filename, global_filename
	filename=a
	global_filename=os.getcwd()+"\\"+a
	with open("venv.txt","w") as f:
		f.write(f"{filename}|{global_filename}|{filename}"+"\\scripts\\python.exe ")
	cmd="python -m venv "+global_filename
	os.system(cmd)
	...
def cre_cmd():
	txt="""import os
with open("venv.txt","r") as f:
	filename,global_filename,py=f.read().split("|")
os.system(filename+r"\\scripts\\activate")
while True:
	
	cmd=input(f"{global_filename}:\\nvenv{{{filename}}}>python ")
	if cmd=="exit":
		import sys;sys.exit()
	cmd=py+cmd
	try:
		os.system(cmd)
	except:
		pass
	
	"""
	with open("cmd.py","w") as f:
		f.write(txt)
if __name__ == '__main__':
    cre()
    cre_cmd()



import hashlib
def md5(text):
    return hashlib.md5(text.encode(encoding='UTF-8')).hexdigest()
def sha256(text):
    return hashlib.new("sha256", text.encode(encoding='UTF-8')).digest()
def sha1(text):
    return hashlib.new("sha1", text.encode(encoding='UTF-8')).digest()



import zmail
"""
content-type: 邮件内容的类型
subject: 邮件主题
to：收件人
from：寄件人
date: 年-月-日 时间 时区
boundary: 如果邮件为multiple parts，你可以得到其分界线
content: 邮件的文本内容（仅在text/plain时可以被解析）
contents: 邮件的body,里面包含着由分界线分割的每一个段落
attachments: None 或者 [['附件名称;编码方式','附件的二进制内容']...]
id: 在邮箱中的id
"""
# 你的邮件内容
def send(to,title,text):
	mail_content = {
		'subject': title,  # 随便填写
		'content_html': f"<pre>{text}</pre>", # Absolute path will be  # 随便填写
	}

	# 使用你的邮件账户名和密码登录服务器
	try:
		server = zmail.server('3450982589@qq.com', 'jdairgsskrhzcjbf')


		server.send_mail(to, mail_content)
	except:
		raise Exception("发太多次邮件啦！！！")
	# 发送邮件

# send("2292068915@qq.com","py发送","hhc")
#

#


class Colorx:
    def __init__(self,color,from_color):
        self.color = color
        self.from_color = from_color
    def _color16_to_rgb(self, color16):
        if color16[0] != "#" or len(color16) != 7:
            raise ValueError("传入的颜色值不符合16进制颜色的标准")
        r16 = "0x"+color16[1:3]
        g16 = "0x"+color16[3:5]
        b16 = "0x"+color16[5:7]
        try:
            r = int(r16,16)
            g = int(g16,16)
            b = int(b16,16)
        except:
            raise ValueError("传入的颜色值不符合16进制颜色的标准")
        return (r,g,b)
    def _rgb_to_color16(self, rgb):
        if len(rgb) != 3 or rgb[0] > 255 or rgb[0] < 0 or rgb[1] > 255 or rgb[1] < 0 or rgb[2] > 255 or rgb[2] < 0:
            raise ValueError("传入的颜色值不符合RGB颜色的标准")
        r = rgb[0]
        g = rgb[1]
        b = rgb[2]
        try:
            r16 = hex(r)
            g16 = hex(g)
            b16 = hex(b)
        except:
            raise ValueError("传入的颜色值不符合RGB颜色的标准")
        r16text = str(r16)[2:4].upper()
        if len(r16text) == 1:
            r16text = "0"+r16text
        g16text = str(g16)[2:4].upper()
        if len(g16text) == 1:
            g16text = "0"+g16text
        b16text = str(b16)[2:4].upper()
        if len(b16text) == 1:
            b16text = "0"+b16text
        return "#"+r16text+g16text+b16text
    def _hsv_to_rgb(self, hsv):
        import math
        if len(hsv) != 3 or hsv[0] > 360 or hsv[0] < 0 or hsv[1] > 1 or hsv[1] < 0 or hsv[2] > 1 or hsv[2] < 0:
            raise ValueError("传入的颜色值不符合HSV颜色的标准")
        h = float(hsv[0])
        s = float(hsv[1])
        v = float(hsv[2])
        h60 = h / 60.0
        h60f = math.floor(h60)
        hi = int(h60f) % 6
        f = h60 - h60f
        p = v * (1 - s)
        q = v * (1 - f * s)
        t = v * (1 - (1 - f) * s)
        r, g, b = 0, 0, 0
        if hi == 0: r, g, b = v, t, p
        elif hi == 1: r, g, b = q, v, p
        elif hi == 2: r, g, b = p, v, t
        elif hi == 3: r, g, b = p, q, v
        elif hi == 4: r, g, b = t, p, v
        elif hi == 5: r, g, b = v, p, q
        r, g, b = int(r * 255), int(g * 255), int(b * 255)
        return r, g, b
    def _rgb_to_hsv(self, rgb):
        import math
        if len(rgb) != 3 or rgb[0] > 255 or rgb[0] < 0 or rgb[1] > 255 or rgb[1] < 0 or rgb[2] > 255 or rgb[2] < 0:
            raise ValueError("传入的颜色值不符合RGB颜色的标准")
        r, g, b = rgb[0]/255.0, rgb[1]/255.0, rgb[2]/255.0
        mx = max(r, g, b)
        mn = min(r, g, b)
        df = mx-mn
        if mx == mn:
            h = 0
        elif mx == r:
            h = (60 * ((g-b)/df) + 360) % 360
        elif mx == g:
            h = (60 * ((b-r)/df) + 120) % 360
        elif mx == b:
            h = (60 * ((r-g)/df) + 240) % 360
        if mx == 0:
            s = 0
        else:
            s = df/mx
        v = mx
        return h, s, v
    def _hsv_to_color16(self, hsv):
        if len(hsv) != 3 or hsv[0] > 360 or hsv[0] < 0 or hsv[1] > 1 or hsv[1] < 0 or hsv[2] > 1 or hsv[2] < 0:
            raise ValueError("传入的颜色值不符合HSV颜色的标准")
        rgb = self._hsv_to_rgb(hsv)
        color16 = self._rgb_to_color16(rgb)
        return color16
    def _color16_to_hsv(self, color16):
        if color16[0] != "#" or len(color16) != 7:
            raise ValueError("传入的颜色值不符合16进制颜色的标准")
        rgb = self._color16_to_rgb(color16)
        hsv = self._rgb_to_hsv(rgb)
        return hsv
    def color_to(self,to_color:"str",change_color:"bool"=False):

        if self.from_color == to_color:
            # print("本来就是原来的格式啦~")
            return self.color

        if self.from_color == "hex" and to_color == "rgb":
            if change_color:
                self.color=self._color16_to_rgb(self.color)
                self.from_color="hex"
            return self._color16_to_rgb(self.color)
        elif self.from_color == "rgb" and to_color == "hex":
            if change_color:
                self.color=self._rgb_to_color16(self.color)
                self.from_color = "rgb"
            return self._rgb_to_color16(self.color)
        elif self.from_color == "hsv" and to_color == "rgb":
            if change_color:
                self.color=self._hsv_to_rgb(self.color)
                self.from_color = "hsv"
            return self._hsv_to_rgb(self.color)
        elif self.from_color == "rgb" and to_color == "hsv":
            if change_color:
                self.color=self._rgb_to_hsv(self.color)
                self.from_color = "rgb"
            return self._rgb_to_hsv(self.color)
        elif self.from_color == "hsv" and to_color == "hex":
            if change_color:
                self.color=self._hsv_to_color16(self.color)
                self.from_color = "hsv"
            return self._hsv_to_color16(self.color)
        elif self.from_color == "hex" and to_color == "rgb":
            if change_color:
                self.color=self._color16_to_hsv(self.color)
                self.from_color="hex"
            return self._color16_to_hsv(self.color)

        else:
            raise ValueError("传入的颜色类型未收录")



"""
开发者：严子昱|崔轩宇

v1.2.20201004

#v1.2.xx以后全是正式版#

#hack-20201004:修复xesuser故障

版权所有，侵权必究

内置函数 导入方式：

```python
from xingyunlib import *
```
### 加密函数s

```python
md=md5("111")#作用：用MD5加密"111"，返回一个字符串
s1=sha1("111")#作用：用sha1加密"111"，返回一个字符串
s256=sha256("111")#作用：用sha1加密"111"，返回一个字符串
```

### 颜色类

```python
color=colorX(颜色,传进去的颜色类型)#类型支持"rgb","hsv","hex"
#示例：
color=colorX((100,100,100),"rgb")
```

#### color.color

返回颜色

```python
z=color.color
```

#### color_to(to_color,change=False)

返回一个要转换的颜色

to_color:转换的颜色，类型支持"rgb","hsv","hex"

change：是否更改原来的颜色

```python
color=colorX((255,255,255),"rgb")#实例化rgb
color.color_to("hex",True)#更改进制为16进制
print(color.color)#打印出color.color
```

"""
def help_xesapp():
	_ver = "2.2.0"
	_mit = "正式版"
	_logo = """
	\   / ┌──   ╭───
	 \ /　Ｅ──  │
	　Ｘ   └──  Ｓ──╮
	 / \  Ａ-Ｐ-Ｐ  │
	/   \       ───╯
	"""

	_pr = """Ver """ + _ver + """ :""" + _mit + """
	""" + _logo
	_help_str = """
		   <xesapp>
	 改-变-xes-作-品-格-局
	开发者：严子昱|崔轩语
	星光版权所有，转载请注明出处
	本库方法命名都是使用下划线命名法


	说明：
	    help()
	        获取帮助
	    help_english()
	        获取英文帮助
	    update()
	        查看更新
	    <xesapp>=xesapp(url)
	        构建一个运行对象
	        url为学而思里面的py作品网址
	    方法：
	    	|---2.0.0---|
		    <xesapp>.get_hot()
		        作用：返回作品的热度\n
		    <xesapp>.get_cover()
		    	作用：返回作品封面的url\n
		    <xesapp>.load_cover(filename)
		    	作用：下载作品封面到filename\n
		    <xesapp>.get_view()
		    	作用：返回作品的观看人数\n
		    <xesapp>.get_code()
		    	作用：返回作品的代码\n
		    <xesapp>.get_hot()
		    	作用：返回作品的热度\n
		    <xesapp>.get_like()
		    	作用：返回作品的点赞数\n
		    <xesapp>.get_unlike()
		    	作用：返回作品的点踩数\n
		    <xesapp>.load_file(cache = "")
		    	作用：默认下载作品源文件到工作目录
		    	如果cache不为空择下载到这个文件夹\n
		    <xesapp>.is_like()
		    	作用：判断在程序运行这个作品有没有人点赞
		    	返回一个bool值，为True，False或None
		    	True：点赞了
		    	False：点踩了
		    	None：没有点赞也没有点踩\n
		    exec(<xesapp>.run_app())
	            作用：运行这个作品(注意要exec里面的内容)
	        |----2.2.0---|
	        <xesapp>.get_published()
	        	作用：获取作品第一次保存时间\n
	        <xesapp>.get_modified()
	        	作用：获取作品最后一次更新时间\n
	        <xesapp>.is_hidden_code()
	        	作用：获取作品是否隐藏代码\n
	        <xesapp>.get_description()
	        	作用：获取作品的说明
			<xesapp>.get_comments()
				作用：获取作品一共有多少人评论
			<xesapp>.is_comments()
				作用：获取从<xesapp>初始化到现在是否有人评论

	---------end----------
	更多方法后续推进中，欢迎大家加入星光工作室
	副室长QQ：2292068915"""

	print(_pr)
	print(_help_str)
def help_xesuser():
	_help_str = """
	<xesuser>
	改-变-xes-作-品-格-局
	开发者：严子昱
	星光版权所有，转载请注明出处
	本库方法命名都是使用下划线命名法
	
	· user_id=get_user_id("作品网址")
		本作品最重要的函数，这个获取作品网址的作者的user_id
	
	· fans_info=get_fans(user_id) #这个user_id从上面拿
		获取这个人粉丝的大部分信息，返回一个列表
		每项是一个字典：
			fans_info[x]["realname"]:获取第x项他的名字
			fans_info[x]["avatar_path"]:获取第x项他头像的url
			fans_info[x]["fans"]:获取第x项他的粉丝数量
			fans_info[x]["follows"]:获取第x项他的关注数量
	
	· follows_info=get_follows(user_id) #大体和get_fans一样
		获取这个人粉丝的大部分信息，返回一个列表
		每项是一个字典：
			follows_info[x]["realname"]:获取第x项他的名字
			follows_info[x]["avatar_path"]:获取第x项他头像的url
			follows_info[x]["fans"]:获取第x项他的粉丝数量
			follows_info[x]["follows"]:获取第x项他的关注数量
	
	· follows_info=get_follows(user_id) #大体和get_fans一样
		获取这个人粉丝的大部分信息，返回一个列表
		每项是一个字典：
			follows_info[x]["realname"]:获取第x项他的名字
			follows_info[x]["avatar_path"]:获取第x项他头像的url
			follows_info[x]["fans"]:获取第x项他的粉丝数量
			follows_info[x]["follows"]:获取第x项他的关注数量
	
	· user_info=get_info(user_id) 
		获取这个人的大部分信息，返回一个字典
		user_info["name"]:返回这个人的名字
		user_info["slogan"]:返回这个人的口号(名字下面那段)
		user_info["fans"]:返回这个人的粉丝数量
		user_info["follows"]:返回这个人的关注数量
		user_info["icon"]:返回这个人的头像url
	
	· <user>=user(user_id)#这个其实大部分都是前面的内容，不过前面的加载比较慢，这个适用需要数据比较多的程序
		<user>.works：获取发布的作品总数
		<user>.work_info：获取发布的作品的信息，返回一个列表，列表的每项都是字典
		<user>.work_num：获取一共有多少个作品（曾经发过的也算）
		
		<user>.fans：获取粉丝总数
		<user>.fans_info：获取粉丝信息（和get_fans返回一样的信息）
		
		<user>.follows：获取关注总数
		<user>.follows_info：获取关注信息（和get_follows返回一样的信息）
		
		<user>.like_num：获取点赞总数
		<user>.view_num：获取浏览总数
		<user>.favorites：获取收藏总数
		
	"""

	print(_help_str)
def help_tkinter_extend():
	print("""
inpu_box：
	创建(就当button控件用)：
	a=inpu_box(tk,函数名,**按钮的配置)
	
	打包：
	a.pack(**pack的配置)
	
	配置：
	a.config_entry(**文本框的配置)
	a.config_button(**按钮的配置)
	其实你基本可以当做Entry来用，除了初始化语句还有配置以外基本都是一样的

""")










"其他api"
import requests,json
def _nice(emoji_str):
	import struct
	return ''.join(
		c if c <= '\uffff' else ''.join(chr(x) for x in struct.unpack('>2H', c.encode('utf-16be'))) for c in emoji_str)
def get_api(http,*w):
	s = requests.Session()
	header = {'Accept': 'application/json, text/plain, */*', 'Accept-Encoding': 'gzip, deflate',
			  'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6',
			  'Cookie': 'xesId=b524835904a4a420cba3dde34890bade; user-select=scratch;  xes_run_token=eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiIuY29kZS54dWVlcnNpLmNvbSIsImF1ZCI6Ii5jb2RlLnh1ZWVyc2kuY29tIiwiaWF0IjoxNjAxODA5NDcxLCJuYmYiOjE2MDE4MDk0NzEsImV4cCI6MTYwMTgyMzg3MSwidXNlcl9pZCI6bnVsbCwidWEiOiJNb3ppbGxhXC81LjAgKFdpbmRvd3MgTlQgMTAuMDsgV2luNjQ7IHg2NCkgQXBwbGVXZWJLaXRcLzUzNy4zNiAoS0hUTUwsIGxpa2UgR2Vja28pIENocm9tZVwvODUuMC40MTgzLjEyMSBTYWZhcmlcLzUzNy4zNiBFZGdcLzg1LjAuNTY0LjY4IiwiaXAiOiIxMTIuNDkuNzIuMTc1In0.9bXcb813GhSPhoUJkezZpV8O50ynm0hhYvszNyczznQ; prelogid=ef8f6d12febabf75bf9599744b73c6f5; xes-code-id=87f66376f1afd34f70339baeca61b7a1.8dbd833da9122d69a17f91054066dbb3; X-Request-Id=82f1c3968c8ff01ee151a0413f56aa84; Hm_lpvt_a8a78faf5b3e92f32fe42a94751a74f1=1601809487',
			  'Host': 'code.xueersi.com', 'Proxy-Connection': 'keep-alive',
			  'Referer': 'http://code.xueersi.com/space/11909587',
			  'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.121 Safari/537.36 Edg/85.0.564.68'}
	total = json.loads(_nice(s.get(http, headers=header).text))
	return total


def get_page(page,lang="",tag="",type="popular",per_page=35):
	return get_api(f"https://code.xueersi.com/api/works/{type}?type={type}&lang={lang}&tag={tag}&page={page}&per_page={per_page}")

#翻译的变量：rcmd
#原始的语句：recommend
#中文含义：推荐
def get_rcmd():
	return get_api("https://code.xueersi.com/api/index/works/modules")
page_all_tags=get_api("https://code.xueersi.com/api/works/all_tags")["data"]









# from calendar import c

import requests
import json


def get_api(http,*w):
	s = requests.Session()
	header = {'Accept': 'application/json, text/plain, */*', 'Accept-Encoding': 'gzip, deflate',
			  'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6',
			  'Cookie': 'xesId=b524835904a4a420cba3dde34890bade; user-select=scratch;  xes_run_token=eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiIuY29kZS54dWVlcnNpLmNvbSIsImF1ZCI6Ii5jb2RlLnh1ZWVyc2kuY29tIiwiaWF0IjoxNjAxODA5NDcxLCJuYmYiOjE2MDE4MDk0NzEsImV4cCI6MTYwMTgyMzg3MSwidXNlcl9pZCI6bnVsbCwidWEiOiJNb3ppbGxhXC81LjAgKFdpbmRvd3MgTlQgMTAuMDsgV2luNjQ7IHg2NCkgQXBwbGVXZWJLaXRcLzUzNy4zNiAoS0hUTUwsIGxpa2UgR2Vja28pIENocm9tZVwvODUuMC40MTgzLjEyMSBTYWZhcmlcLzUzNy4zNiBFZGdcLzg1LjAuNTY0LjY4IiwiaXAiOiIxMTIuNDkuNzIuMTc1In0.9bXcb813GhSPhoUJkezZpV8O50ynm0hhYvszNyczznQ; prelogid=ef8f6d12febabf75bf9599744b73c6f5; xes-code-id=87f66376f1afd34f70339baeca61b7a1.8dbd833da9122d69a17f91054066dbb3; X-Request-Id=82f1c3968c8ff01ee151a0413f56aa84; Hm_lpvt_a8a78faf5b3e92f32fe42a94751a74f1=1601809487',
			  'Host': 'code.xueersi.com', 'Proxy-Connection': 'keep-alive',
			  'Referer': 'http://code.xueersi.com/space/11909587',
			  'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.121 Safari/537.36 Edg/85.0.564.68'}
	total = json.loads(_nice(s.get(http, headers=header).text))
	return total
def _nice(emoji_str):
	import struct
	return ''.join(
		c if c <= '\uffff' else ''.join(chr(x) for x in struct.unpack('>2H', c.encode('utf-16be'))) for c in emoji_str)
def get_fans_info(id, lengh=None):
	id = str(id)
	# page = 1
	if lengh == None:
		lengh = (int(get_info(id)["fans"])//150)+1
	c=[]
	for x in range(lengh):
		# headers = {'Content-Type': 'application/json'}
		total = get_api(f"http://code.xueersi.com/api/space/fans?user_id={id}&page={str(x+1)}&per_page=150")
		c+=total["data"]["data"]

	return c
def get_follows_info(id, lengh=None):
	id = str(id)
	page = 1
	if lengh == None:
		lengh = get_info(id)["follows"]

	total = get_api(
		"http://code.xueersi.com/api/space/follows?user_id=" + id + "&page=" + str(page) + "&per_page=" + str(lengh))
	return total["data"]["data"]
def get_info(id):
	s = requests.Session()
	headers = {'Accept': 'application/json, text/plain, */*', 'Accept-Encoding': 'gzip, deflate', 'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6', 'Cookie': 'xesId=b524835904a4a420cba3dde34890bade; user-select=scratch;  xes_run_token=eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiIuY29kZS54dWVlcnNpLmNvbSIsImF1ZCI6Ii5jb2RlLnh1ZWVyc2kuY29tIiwiaWF0IjoxNjAxODA5NDcxLCJuYmYiOjE2MDE4MDk0NzEsImV4cCI6MTYwMTgyMzg3MSwidXNlcl9pZCI6bnVsbCwidWEiOiJNb3ppbGxhXC81LjAgKFdpbmRvd3MgTlQgMTAuMDsgV2luNjQ7IHg2NCkgQXBwbGVXZWJLaXRcLzUzNy4zNiAoS0hUTUwsIGxpa2UgR2Vja28pIENocm9tZVwvODUuMC40MTgzLjEyMSBTYWZhcmlcLzUzNy4zNiBFZGdcLzg1LjAuNTY0LjY4IiwiaXAiOiIxMTIuNDkuNzIuMTc1In0.9bXcb813GhSPhoUJkezZpV8O50ynm0hhYvszNyczznQ; prelogid=ef8f6d12febabf75bf9599744b73c6f5; xes-code-id=87f66376f1afd34f70339baeca61b7a1.8dbd833da9122d69a17f91054066dbb3; X-Request-Id=82f1c3968c8ff01ee151a0413f56aa84; Hm_lpvt_a8a78faf5b3e92f32fe42a94751a74f1=1601809487', 'Host': 'code.xueersi.com', 'Proxy-Connection': 'keep-alive', 'Referer': 'http://code.xueersi.com/space/11909587', 'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.121 Safari/537.36 Edg/85.0.564.68'}

	total = json.loads(
		_nice(s.get("http://code.xueersi.com/api/space/profile?user_id=" + str(id), headers=headers).text))[
		"data"]
	return {
		# "user_id": total["user_id"],
		"name": total["realname"],
		"slogan": total["signature"],
		"fans": total["fans"],
		"follows": total["follows"],
		"icon": total["avatar_path"]
	}
def get_user_id(id):
	id = id.split("&pid=")[1].split("&")[0]
	url = "http://code.xueersi.com/api/compilers/" + id + "?id=" + id
	a = get_api(url)
	# self._nice()
	return a["data"]["user_id"]
class user:
	def __init__(self, id):
		s = requests.Session()
		id = str(id)
		url = "http://code.xueersi.com/api/space/index?user_id=" + id
		headers ={'Accept': 'application/json, text/plain, */*', 'Accept-Encoding': 'gzip, deflate', 'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6', 'Cookie': 'xesId=b524835904a4a420cba3dde34890bade; user-select=scratch; userGrade=8; wx=408f3d126b6e5da40f1231f4a8e82cecmqx0f94f4q; Hm_lvt_a8a78faf5b3e92f32fe42a94751a74f1=1600492811,1600492816; xes-code-id=87f66376f1afd34f70339baeca61b7a1.8dbd833da9122d69a17f91054066dbb3; prelogid=82f1c3968c8ff01ee151a0413f56aa84; xes_run_token=eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiIuY29kZS54dWVlcnNpLmNvbSIsImF1ZCI6Ii5jb2RlLnh1ZWVyc2kuY29tIiwiaWF0IjoxNjAxODA5NDg4LCJuYmYiOjE2MDE4MDk0ODgsImV4cCI6MTYwMTgyMzg4OCwidXNlcl9pZCI6bnVsbCwidWEiOiJNb3ppbGxhXC81LjAgKFdpbmRvd3MgTlQgMTAuMDsgV2luNjQ7IHg2NCkgQXBwbGVXZWJLaXRcLzUzNy4zNiAoS0hUTUwsIGxpa2UgR2Vja28pIENocm9tZVwvODUuMC40MTgzLjEyMSBTYWZhcmlcLzUzNy4zNiBFZGdcLzg1LjAuNTY0LjY4IiwiaXAiOiIxMTIuNDkuNzIuMTc1In0.Depgg9J-Hbe5RDeQvQwn59Aj0aa4CnndKeOKad-5WTY; X-Request-Id=db987fe27cbc7c51525a99c6419a34c7; Hm_lpvt_a8a78faf5b3e92f32fe42a94751a74f1=1601810095', 'Host': 'code.xueersi.com', 'Proxy-Connection': 'keep-alive', 'Referer': 'http://code.xueersi.com/space/11909587', 'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.121 Safari/537.36 Edg/85.0.564.68'}
		a = s.get(url=url, headers=headers)
		a = json.loads(_nice(a.text))
		a = a["data"]
		self.data = a
		self.works = a["works"]["total"]
		self.fans = a["fans"]["total"]
		self.follows = a["follows"]["total"]
		self.overview = a["overview"]
		self.like_num = self.overview["likes"]
		self.view_num = self.overview["views"]
		self.work_num = self.overview["works"]
		self.favorites = self.overview["favorites"]
#nice
#
# # a = user(get_user_id(
# # 	"http://code.xueersi.com/home/project/detail?lang=code&pid=6076813&version=offline&form=python&langType=python"))
# # print(a.work_info)


import hashlib,pickle

def md5(text):

	return hashlib.md5(text.encode(encoding='UTF-8')).hexdigest()
class user_login:
	def __init__(self,filename=None):
		if filename!=None:
			with open(filename+".pickle","rb") as file:
				c = pickle.load(file)
				# print(c)
				self.name = c[0]
				self.key = c[1]
		else:
			self.key = None
			self.name = None
	def registered(self,name,key,flag=True,filename=None):
		if (self.key != None and self.name != None)and flag:
			return False
		else:
			with open(filename+".pickle","wb") as file:
				fp = [name, md5(key)]
				pickle.dump(fp, file)
				self.key=md5(key)
				self.name=name
			return True
	def login(self,name,key):
		if self.key==md5(key) and self.name==name :
			return True
		elif self.key == None and self.name == None:
			return None
		else:
			return False
	def load(self,filename):
		with open(filename + ".pickle", "rb") as file:
			c = pickle.load(file)
			# print(c)
			self.name = c[0]
			self.key = c[1]








def get_interpreter():
	import sys
	return sys.executable

def get_pakages():
	import os
	import re
	# http://127.0.0.1:55820/package/search?name=xingyunlib
	interpreter_pos = get_interpreter()
	# print(os.getcwd())
	z = os.popen(interpreter_pos + " -m pip list")
	a = z.read()
	z.close()
	c = a.split("\n")[2:]
	num = []
	for x in c:
		x = re.split("\s", x)
		while "" in x:
			x.remove("")
		if x == []:
			continue
		num.append(x)
	return num


def find_pakages(pakage, ver=False):
	Ls = get_pakages()
	for x in Ls:
		if x[0] == pakage:
			if ver:
				if x[1] == ver:
					return True
				else:
					return x[1]
			return True
	return False


def upgrade_pakages(pakage):
	import os
	if type(pakage) != type(" "):
		pakage = "".join(pakage)
	os.system(get_interpreter() + " -m pip install --upgrade " + pakage)


def install_pakages(pakage, interpreter=None):
	import os
	if type(pakage) != type(" "):
		pakage = " ".join(pakage)
	if interpreter == None:
		interpreter = get_interpreter()
	print(pakage)
	os.system(interpreter + " -m pip install " + pakage + " -i https://pypi.douban.com/simple")


def dump_pip_list(filename):
	with open(filename + ".pip", "w", encoding="utf-8")as f:
		f.write(str(get_pakages()))


def load_pip_list(filename, interpreter=None):
	with open(filename + ".pip", "r", encoding="utf-8")as f:
		z = eval(f.read())
		# print(z)
		b = []
		for x in z:
			b.append(x[0])
		install_pakages(b, interpreter=interpreter)

# print(find_packages("xingyunlib"))
# exit()


import pygame,sys

def print_text(font,text,x,y,color=(255,255,255),target=None):
    imgText =font.render(text, True, color)
    if target == None:
        target = pygame.display.get_surface()
    target.blit(imgText, (x,y))

def icon(name,x,y,screen=None):
    if x>y:
        c=(x-y)//2
        aaa=0
        kfd=y
    elif y>x:
        aaa=(y-x)//2
        c=0
        kfd=x
    else:
        aaa=0
        c=0
        kfd=x
    if screen==None:
        screen=pygame.display.get_surface()
    a=pygame.image.load(name)
    a=pygame.transform.smoothscale(a, (kfd, kfd))
    k=0
    flag=True
    while flag:
        k+=1
        screen.fill((255,255,255))
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            elif event.type == pygame.KEYDOWN:
                flag=False
        screen.blit(a,(c,aaa))
        pygame.display.update()

class Mouse:
    def __init__(self,size,apple,type_do="rect",up_color=(255,255,255),down_color=(0,255,0)):
        self.down_color=down_color
        self.up_color=up_color
        self.size=size
        self.rect=pygame.Rect(0,0,size,size)
        self.type=type_do
        self.apple=apple
    def update(self,screen):
        self.rect.centerx,self.rect.centery=pygame.mouse.get_pos()
        down=pygame.mouse.get_pressed()[0]
        if self.type=="rect":
            if down:
                pygame.draw.rect(screen,self.down_color,self.rect,self.apple+2)
            else:
                pygame.draw.rect(screen,self.up_color,self.rect,self.apple)
        else:
            if down:
                pygame.draw.ellipse(screen,self.down_color,self.rect,self.apple+2)
            else:

                pygame.draw.ellipse(screen,self.up_color,self.rect,self.apple)
    def get_rect(self):
        self.rect.center=pygame.mouse.get_pos()
        return self.rect

class Sound:
    #初始化
    def __init__(self,file,cha=True):
        self.sound=pygame.mixer.Sound(file)
        self.channal=pygame.mixer.find_channel(cha)
    #播放
    def play(self):
        self.channal.play(self.sound)
    #停止
    def stop(self):
        self.sound.stop()

class Point(object):
    #初始化
    def __init__(self,x,y):
        self._x=x
        self._y=y
    #x
    def get_x(self):
        return self._x
    def set_x(self,x):
        self._x=x
    #y
    def get_y(self):
        return self._y
    def set_y(self,y):
        self._y=y
    #检测碰撞
    def colliderect(self,rect):
        return rect.collidepoint(self._x,self._y)
    # 放回值
    def __str__(self):
        return "{x:{:.0f}".format(self._x)+",y:{:.0f}".format(self._y)+"}"





from PIL import Image



def resize(imgfile,xy):
	Image.open(imgfile).resize(
		xy, Image.ANTIALIAS).save(imgfile)

def cloud(text,font_path,savafile=None,image=None,background_color="white",color=True):
    import PIL.Image
    import numpy as np
    import wordcloud
    cut_text = "".join(text)
    if image!=None:
        background_image = np.array(PIL.Image.open(image))
        c = wordcloud.WordCloud(
            font_path=font_path,
            background_color=background_color,
            mask=background_image).generate(cut_text)
    else:
        c = wordcloud.WordCloud(
            font_path=font_path,
            background_color=background_color).generate(cut_text)
    if color and image!=None:
        image_colors = wordcloud.ImageColorGenerator(background_image)
        c=c.recolor(color_func=image_colors)
    # print(type(c))
    if savafile!=None:
        c.to_file(savafile)
    return c


def add_text(img, text, font: "tuple" = ('C:/windows/fonts/Dengl.ttf', 20), XY=(0, 0), fillColor="#000000",
             show_img: "bool" = False, change_name=None):
    import PIL.ImageDraw
    import PIL.ImageFont
    setFont = PIL.ImageFont.truetype(*font)

    # img=imglist[x]
    image = Image.open(img)
    draw = PIL.ImageDraw.Draw(image)
    width, height = image.size
    if type(XY) == type(""):
        X, Y = eval(XY)
        XY = (X, Y)
    draw.text(XY, text, font=setFont, fill=fillColor)
    if change_name != None:
        img = change_name
    image.save(img)

    if show_img:
        image.show()


#!/usr/bin/env python
# coding=utf-8

from setuptools import setup, find_packages

setup(
    name='xingyunlib',
    version="1.2.20201024",
    description=(
        '星光☆学而思专用库'
    ),
    long_description=open('README.md',encoding="utf-8").read(),
    long_description_content_type='text/markdown',
    author='严子昱',
    author_email='yanziyu252625@qq.com',
    maintainer='严子昱',
    maintainer_email='yanziyu252625@qq.com',
    license='BSD License',
    packages=find_packages(),
    platforms=["all"],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[
        'requests',"pygame","zmail","pillow","numpy"#,"alsolib"
    ]
)
#
# 9abe5372d54460c07f202585e7da2cf53cf016bc884542732e07a3544275


"""
## Data

初始化方法：

```python
dta=data.Data(filename,find=False)
\"\"\"
filename:文件名，若没有则创建
find:搜索模式，若没有filename则报错
\"\"\"
```

输入数据：

### change_data(data)

```python
dta=data.Data(filename,find=False)
#略
\"\"\"
data参数支持的数据:
list,bool,tutle,str,set,dict,int,float
\"\"\"
dta.change_data("加油！")
```
### eval_data()

把从文件中保存的数据转换成可执行内容

```python
\"\"\"fruit.txt内容：
["apple","orange"]
\"\"\"
dta=data.Data("fruit.txt",find=False)
dta.eval_data()
for x in dta.data:
    print(f"fruit:{x}")
\"\"\"
输出结果：
fruit:apple
fruit:orange
\"\"\"


```

### save()

保存结果到文件中

```python
dta=data.Data("great",find=False)
dta.change_data(["hhc","hhv"])
dta.save()
\"\"\"great内容:
["hhc","hhv"]\"\"\"
```
"""
class Data(object):
	def __init__(self,initfile="",find=False):
		if initfile=="":
			raise FileExistsError("未指定打开的文件")
		try:
			if find==False:
				try:
					a = open(initfile, "r")
					self.data = eval(a.read())
				except:
					a = open(initfile,"w")
					self.data = eval(a.read())
			else:
				a = open(initfile, "r")
				self.data = eval(a.read())
		except:
			raise FileNotFoundError("未能找到要打开的文件")
		self.file=initfile
	def eval_load(self):
		try:
			self.data=eval(self.data)
			return self.data
		except:
			return self.data
	def change_data(self,w):
		self.data=w
	def save(self):
		with open(self.file,"w") as f:
			try:
				f.write(self.data)
			except:
				try:
					f.write(str(self.data))
				except:
					# 翻译的变量：FileExistsError
					# 原始的语句：File Exists Error
					# 中文含义：文件存在错误
					raise IOError("无法保存此数据！")
	def reload(self,initfile="",find=False):
		if initfile=="":
			raise FileExistsError("未指定打开的文件")
		try:
			if find==False:
				try:
					a = open(initfile, "r")
					self.data = eval(a.read())
				except:
					a = open(initfile,"w")
					self.data = eval(a.read())
			else:
				a = open(initfile, "r")
				self.data = eval(a.read())
		except:
			raise FileNotFoundError("未能找到要打开的文件")
		self.file=initfile
class Key:
	def __init__(self,filename=None):
		if filename!=None:
			try:
				with open(filename,"r") as f:
					self.encode=f.read()
			except:
				self.encode = filename
		else:
			self.encode=None
			self.decode=None
	def decoding(self,key,v=2):
		s=list(self.encode)
		for x in range(len(self.encode)):
			for y in list(key):
				if x % v==0:
					s[x] = chr(abs(ord(s[x])-ord(y)))
				else:
					s[x] = chr(abs(ord(s[x]) + ord(y)))
		self.decode="".join(s)
		return s
	def encoding(self,txt,key,v=2):
		s = list(txt)
		for x in range(len(txt)):
			for y in list(key):
				if x % v == 0:
					s[x] = chr(abs(ord(s[x]) + ord(y)))
				else:
					s[x] = chr(abs(ord(s[x]) - ord(y)))
		self.encode = "".join(s)
		self.decode = txt
		return s
	def sava_encode(self,filename):
		with open(filename,"w") as f:
			f.write(self.encode)